@@ -53,16 +53,6 @@ typedef struct {
 
 typedef struct {
 	GObjectClass parent;
-
-	/* Signals */
-	void (*connection_activated) (NMVPNManager *manager,
-	                              NMVPNConnection *connection);
-
-	void (*connection_deactivated) (NMVPNManager *manager,
-	                                NMVPNConnection *connection,
-	                                NMVPNConnectionState new_state,
-	                                NMVPNConnectionState old_state,
-	                                NMVPNConnectionStateReason reason);
 } NMVPNManagerClass;
 
 GType nm_vpn_manager_get_type (void);
@@ -78,19 +68,7 @@ NMActiveConnection *nm_vpn_manager_activate_connection (NMVPNManager *manager,
                                                         GError **error);
 
 gboolean nm_vpn_manager_deactivate_connection (NMVPNManager *manager,
-                                               const char *path,
+                                               NMVPNConnection *connection,
                                                NMVPNConnectionStateReason reason);
 
-void nm_vpn_manager_add_active_connections (NMVPNManager *manager,
-                                            NMConnection *filter,
-                                            GPtrArray *list);
-
-GSList *nm_vpn_manager_get_active_connections (NMVPNManager *manager);
-
-NMVPNConnection *nm_vpn_manager_get_vpn_connection_for_active (NMVPNManager *manager,
-                                                               const char *active_path);
-
-NMConnection *nm_vpn_manager_get_connection_for_active (NMVPNManager *manager,
-                                                        const char *active_path);
-
 #endif /* NM_VPN_MANAGER_H */
